/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <ppcboot.h>
#include <mpc8240.h>
#include <asm/processor.h>
#include "speed.h"

/*use UART2 to measure bus speed*/
#include "ns16550.h"


/* Access functions for the Machine State Register */
static __inline__ unsigned long get_msr(void)
{
    unsigned long msr;

    __asm__ ("mfmsr %0" : "=r" (msr) :);
    return msr;
}

static __inline__ void set_msr(unsigned long msr)
{
    __asm__ __volatile__("mtmsr %0" : : "r" (msr));
}

/* ------------------------------------------------------------------------- */

ulong get_bus_freq (ulong ignore)
{
    return 66666666;
}

/* ------------------------------------------------------------------------- */

/*
 * Measure CPU clock speed
 */

/*
 * table to convert pllratio to actual processor clock scaling factor (*2)
 * if you are using a a different processor with your sandpoint,
 * have a look at pmc.c in the dink source for values, or
 * figure it out from the hardware book
 */

#ifdef CONFIG_MPC8240
const unsigned char pllratio_to_factor[32] = {
    0, 0, 0, 2, 4, 4, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 2, 0, 0, 0, 9, 6, 0, 8, 0, 0, 0, 7, 0
};
#else
  #error Enter clock divider array here!
#endif

ulong get_gclk_freq (void)
{
   uint hid1;
   hid1 = mfspr(HID1);
   #ifdef CONFIG_MPC8240
   hid1 = (hid1 >> (32-5)) & 0x1f; /* 5 bits for PLL ration on 8240*/
   #else
   hid1 = (hid1 >> (32-4)) & 0xf; /* 4 bits on everythings else*/
   #endif

   return pllratio_to_factor[hid1] * get_bus_freq(0)/2;
}

/* ------------------------------------------------------------------------- */

